@@ -1,12 +1,15 @@
+import sys
+import contextlib
 from distutils.errors import DistutilsOptionError
 from unittest import TestLoader
-import sys
+
+from setuptools.extern import six
+from setuptools.extern.six.moves import map
 
 from pkg_resources import (resource_listdir, resource_exists, normalize_path,
                            working_set, _namespace_packages,
                            add_activation_listener, require, EntryPoint)
 from setuptools import Command
-from setuptools.compat import PY3
 from setuptools.py31compat import unittest_main
 
 
@@ -41,6 +44,17 @@ class ScanningLoader(TestLoader):
             return tests[0]  # don't create a nested suite for only one return
 
 
+# adapted from jaraco.classes.properties:NonDataProperty
+class NonDataProperty(object):
+    def __init__(self, fget):
+        self.fget = fget
+
+    def __get__(self, obj, objtype=None):
+        if obj is None:
+            return self
+        return self.fget(obj)
+
+
 class test(Command):
     """Command to run unit tests after in-place build"""
 
@@ -78,7 +92,7 @@ class test(Command):
         if self.test_runner is None:
             self.test_runner = getattr(self.distribution, 'test_runner', None)
 
-    @property
+    @NonDataProperty
     def test_args(self):
         return list(self._test_args())
 
@@ -89,7 +103,15 @@ class test(Command):
             yield self.test_suite
 
     def with_project_on_sys_path(self, func):
-        with_2to3 = PY3 and getattr(self.distribution, 'use_2to3', False)
+        """
+        Backward compatibility for project_on_sys_path context.
+        """
+        with self.project_on_sys_path():
+            func()
+
+    @contextlib.contextmanager
+    def project_on_sys_path(self):
+        with_2to3 = six.PY3 and getattr(self.distribution, 'use_2to3', False)
 
         if with_2to3:
             # If we run 2to3 we can not do this inplace:
@@ -124,7 +146,7 @@ class test(Command):
             working_set.__init__()
             add_activation_listener(lambda dist: dist.activate())
             require('%s==%s' % (ei_cmd.egg_name, ei_cmd.egg_version))
-            func()
+            yield
         finally:
             sys.path[:] = old_path
             sys.modules.clear()
@@ -141,15 +163,17 @@ class test(Command):
         cmd = ' '.join(self._argv)
         if self.dry_run:
             self.announce('skipping "%s" (dry run)' % cmd)
-        else:
-            self.announce('running "%s"' % cmd)
-            self.with_project_on_sys_path(self.run_tests)
+            return
+
+        self.announce('running "%s"' % cmd)
+        with self.project_on_sys_path():
+            self.run_tests()
 
     def run_tests(self):
         # Purge modules under test from sys.modules. The test loader will
         # re-import them from the build location. Required when 2to3 is used
         # with namespace packages.
-        if PY3 and getattr(self.distribution, 'use_2to3', False):
+        if six.PY3 and getattr(self.distribution, 'use_2to3', False):
             module = self.test_suite.split('.')[0]
             if module in _namespace_packages:
                 del_modules = []
